# encoding: UTF-8
=begin
Created on 2015-04-08

@author: Nikolay Moskvin <moskvin@sibext.com>

=end

module Croket
  module Rails
    module ActionControllerCatcher

      def self.included(base)
        if base.method_defined?(:rescue_action_in_public)
          base.send(:alias_method, :rescue_action_in_public_without_croket, :rescue_action_in_public)
          base.send(:alias_method, :rescue_action_in_public, :rescue_action_in_public_with_croket)
        end
      end

      KINDS = [:commit, :rollback].freeze

      def run_callbacks(kind, *args, &block)
        if KINDS.include?(kind)
          begin
            super
          rescue StandardError => exception
            # This exception will NOT be escalated, so notify it here.
            Croket.notify(exception, nil)
            raise
          end
        else
          # Let the post process handle the exception
          super
        end
      end

      private

      # Overrides the rescue_action method in ActionController::Base, but does not inhibit
      # any custom processing that is defined with Rails 2's exception helpers.
      def rescue_action_in_public_with_hahulya(exception)
        error_id = Croket.notify(exception, hahulya_request_data)
      end

    end
  end
end

